% This code computes the Bayesian estimation of the "encompassing model" in Chang and Fernandez (2010)
% On the Sources of Aggregate Fluctuations in Emerging Economies. NBER WP
% 15938. April, 2010.

function [out,parnew,par,VarCov] = encompassing_estimation(nsim,V,DATA) % To be used in the first or final iteration 
t0=clock;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1 ROUTINE: COMPUTING PRIORS 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parinit = ...
  [0.8937 0.7192 0.8054 0.6877 0.0066 0.0012 0.0042 0.7282 14.7752 0.0064 0.0113 0.0304 0.0078]; %using posterior mode 

%                            par(1)    par(2) 
RHOa_Pr     = [parinit(1,1) 356.2000  18.753];   % Beta pdf with mean 0.95 and s.d. 0.0112
RHOg_Pr     = [parinit(1,2) 285.1250 110.8819];  % Beta pdf with mean 0.72 and s.d. 0.0225
RHOr_Pr     = [parinit(1,3)  44.2608   9.0655];  % Beta pdf with mean 0.83 and s.d. 0.0510
TETA_Pr     = [parinit(1,4)   2.0000   2.0000];  % Beta pdf with mean 0.50 and s.d. 0.2240
SIGa_Pr     = [parinit(1,5)   2.0603   0.0036];  % Gamma pdf with mean 0.0074 and s.d. 0.0056 
SIGg_Pr     = [parinit(1,6)   2.0603   0.0036];  % Gamma pdf with mean 0.0074 and s.d. 0.0056
SIGr_Pr     = [parinit(1,7)   5.5524   0.0013];  % Gamma pdf with mean 0.0070 and s.d. 0.0031
ETA_Pr      = [parinit(1,8)  99.2176   0.0101];  % Gamma pdf with mean 1.0000 and s.d. 0.1010
PHI_Pr      = [parinit(1,9)   3.0000   2.0000];  % Gamma pdf with mean 6.0000 and s.d. 3.4600
SIGY_Pr     = [parinit(1,10)  4.0000   0.005];   % Gamma pdf with mean 0.0200 and s.d. 0.0100
SIGC_Pr     = [parinit(1,11)  4.0000   0.005];   % Gamma pdf with mean 0.0200 and s.d. 0.0100
SIGI_Pr     = [parinit(1,12)  4.0000   0.005];   % Gamma pdf with mean 0.0200 and s.d. 0.0100
SIGNX_Pr    = [parinit(1,13)  4.0000   0.005];   % Gamma pdf with mean 0.0200 and s.d. 0.0100

MATPRIORS = [RHOa_Pr; RHOg_Pr; RHOr_Pr; TETA_Pr; SIGa_Pr; SIGg_Pr; SIGr_Pr; ...
             ETA_Pr;  PHI_Pr;  SIGY_Pr; SIGC_Pr; SIGI_Pr; SIGNX_Pr];
      
logprior(1)=logprcalc(MATPRIORS); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2 ROUTINE:  KALMAN FILTER
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[gx,hx,ind] = modelsolve(parinit); %Solving Model
 
if ind == 1 % ind. is a flag signaling the multiple solutions or no solution
   return;
end

PHI_MAT = [hx(1:11,1:11),zeros(11,4);...
           gx(1,1:11)   ,zeros(1,4);...
           gx(2,1:11)   ,zeros(1,4);...
           gx(3,1:11)   ,zeros(1,4);...
           gx(4,1:11)   ,zeros(1,4)];

A_MAT = [log(1.006);log(1.006);log(1.006);0]; %Builbing the first measurement eq. Matrix (with constants)
GAMMA_MAT = [zeros(4,11) eye(4)]; %Builbing the second measurement eq. Matrix
BMAT = zeros(15,3); BMAT(3,1) = 1; BMAT(4,2) = 1; BMAT(5,3) = 1;

DATA_T =DATA';

SIG1 = diag([SIGg_Pr(1,1)^2 SIGr_Pr(1,1)^2 SIGa_Pr(1,1)^2],0); %Variance of Structural Shocks
RMAT = zeros(4,4);
RMAT(1,1) = SIGY_Pr(1,1)^2;  % Variance of Measurement eq. Error in Y
RMAT(2,2) = SIGC_Pr(1,1)^2;  % Variance of Measurement eq. Error in C
RMAT(3,3) = SIGI_Pr(1,1)^2;  % Variance of Measurement eq. Error in I
RMAT(4,4) = SIGNX_Pr(1,1)^2; % Variance of Measurement eq. Error in NX

logdata(1) = LLcalc_constant3_me(DATA_T,A_MAT,GAMMA_MAT,PHI_MAT,SIG1,BMAT,RMAT);
logpost=logprior(1)+logdata(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3 METROPOLIS-HASTINGS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

npars = 13; %Number of parameters. 
k = 1;
nacc=0;
x = mvnrnd(zeros(1,npars),V,nsim);
nind=0;
par=zeros(nsim,npars);
partrans=zeros(nsim,npars);
logprior=zeros(nsim,1);
par(1,:)=parinit;

while k<nsim    
    [parnew,partransnew]=rwstep(par(k,:),x(k,:));
     
    RHOa_Pr     = [parnew(1) 356.2000  18.753]; 
    RHOg_Pr     = [parnew(2) 285.1250 110.8819]; 
    RHOr_Pr     = [parnew(3)  44.2608   9.0655]; 
    TETA_Pr     = [parnew(4)   2.0000   2.0000]; 
    SIGa_Pr     = [parnew(5)   2.0603   0.0036]; 
    SIGg_Pr     = [parnew(6)   2.0603   0.0036]; 
    SIGr_Pr     = [parnew(7)   5.5524   0.0013]; 
    ETA_Pr      = [parnew(8)  99.2176   0.0101]; 
    PHI_Pr      = [parnew(9)   3.0000   2.0000]; 
    SIGY_Pr     = [parnew(10)  4.0000   0.005]; 
    SIGC_Pr     = [parnew(11)  4.0000   0.005]; 
    SIGI_Pr     = [parnew(12)  4.0000   0.005]; 
    SIGNX_Pr    = [parnew(13)  4.0000   0.005]; 
    
    MATPRIORS = [RHOa_Pr; RHOg_Pr; RHOr_Pr; TETA_Pr; SIGa_Pr; SIGg_Pr; SIGr_Pr; ...
                 ETA_Pr;  PHI_Pr;  SIGY_Pr; SIGC_Pr; SIGI_Pr; SIGNX_Pr];
    
    lprnew=logprcalc(MATPRIORS);
    
    [new_gx,new_hx,ind] = modelsolve(parnew);%replace weight with new draw
     
    if ind == 1
         x(k,:)=mvnrnd(zeros(1,npars),V);
         nind=nind+1;
        continue;
    else               
              
           new_PHI_MAT = [new_hx(1:11,1:11),zeros(11,4);...
           new_gx(1,1:11)    ,zeros(1,4);...
           new_gx(2,1:11)    ,zeros(1,4);...
           new_gx(3,1:11)    ,zeros(1,4);...
           new_gx(4,1:11)    ,zeros(1,4)];

           new_RMAT = zeros(4,4);
           new_RMAT(1,1) = SIGY_Pr(1,1)^2; 
           new_RMAT(2,2) = SIGC_Pr(1,1)^2; 
           new_RMAT(3,3) = SIGI_Pr(1,1)^2; 
           new_RMAT(4,4) = SIGNX_Pr(1,1)^2;            
    end    

    k=k+1;
    
    if mod(k,500)==0
        k
    end

    SIG = diag([SIGg_Pr(1,1)^2 SIGr_Pr(1,1)^2 SIGa_Pr(1,1)^2],0);
    logdnew = LLcalc_constant3_me(DATA_T,A_MAT,GAMMA_MAT,new_PHI_MAT,SIG,BMAT,new_RMAT);
    logpostnew=lprnew+logdnew;
    ratio=exp(logpostnew-logpost);
    accprob=min(1,ratio);
    lp=rand(1,1);
    if(lp<=accprob)
        par(k,:)=parnew;
        partrans(k,:)=partransnew;
        logprior(k)=lprnew;
        logdata(k)=logdnew;
        logpost=logpostnew;
        nacc=nacc+1;
    else
        par(k,:)=par(k-1,:);
        partrans(k,:)=partrans(k-1,:);
        logprior(k)=logprior(k-1);
        logdata(k)=logdata(k-1);
    end
end

VarCov = cov(partrans);
out.parsnew = parnew;
out.pars=par;
out.partrans=partrans;
out.logprior=logprior;
out.logdata=logdata;
out.logpost=logpost;
fprintf(1,'Iteration %8.0f\n',k);
fprintf(1,'Number of rejections so far %20.0f\n',nind);
t1=etime(clock,t0);
fprintf(1,'Number of Seconds used: %8.2f secs\n',t1);
fprintf(1,'Average time per iteration:%5.2f secs\n',t1/k);
fprintf(1,'Acceptance Prob: %5.4f\n',nacc/nsim);
